require 'active_model'

module OpenvasCli
  class VasPeriod
    include ActiveModel::Validations
    include ActiveModel::Dirty
    include Comparable
    
    attr_reader :number
    attr_reader :period
    
    define_attribute_methods [:number, :period]
    
    validates :number, :numericality => { :greater_than_or_equal_to => 0 }
    validates :period, :inclusion => [:second, :minute, :hour, :day, :month, :year]
    
    def initialize(params={})
      @number = params[:number] if params[:number]
      @period = params[:period] if params[:period]
    end
    
    def number=(v)
      number_will_change! unless @number == v
      @number = v
    end
    
    def period=(v)
      period_will_change! unless @period == v
      @period = v
    end
    
    def <=> (rhs)
      to_seconds - rhs.to_seconds
    end
    
    def + (rhs)
      if rhs.kind_of? VasPeriod
        VasPeriod.from_seconds(to_seconds + rhs.to_seconds)
      else
        VasPeriod.from_seconds(to_seconds + rhs)
      end
    end
    
    def - (rhs)
      if rhs.kind_of? VasPeriod
        VasPeriod.from_seconds(to_seconds - rhs.to_seconds)
      else
        VasPeriod.from_seconds(to_seconds - rhs)
      end
    end
    
    def to_seconds
      case @period
      when :year
        @number * 365.25 * 86400
      when :month
        @number * 365.25 * 86400 / 12
      when :day
        @number * 86400
      when :hour
        @number * 3600
      when :minute
        @number * 60
      else
        @number
      end
    end

    def self.from_seconds(seconds)
      if seconds % 86400 == 0
        VasPeriod.new(:number => seconds / 86400, :period => :day)
      elsif seconds % 3600 == 0
        VasPeriod.new(:number => seconds / 3600, :period => :hour)
      elsif seconds % 60 == 0
        VasPeriod.new(:number => seconds / 60, :period => :minute)
      else
        VasPeriod.new(:number => seconds, :period => :second)
      end
    end 
    
    def self.from_months(months)
      if months % 12 == 0
        VasPeriod.new(:number => months / 12, :period => :month)
      else
        VasPeriod.new(:number => months, :period => :month)
      end
    end
  end
end
