require 'vas_base'
require 'vas_result'

module OpenvasCli
  class VasTask < VasBase
    attr_reader   :name
    attr_reader   :comment
    attr_accessor :status
    attr_accessor :times_run
    attr_accessor :last_report_id
    attr_reader   :config_id
    attr_reader   :target_id
    attr_reader   :schedule_id
    
    validates :id, :UUID => true, :if => Proc.new { |t| t.id }
    validates :config_id, :UUID => true, :if => Proc.new { |t| t.config_id }
    validates :target_id, :UUID => true, :if => Proc.new { |t| t.target_id }
    validates_with ImmutableChildrenValidator
    
    
    define_attribute_methods [:name, :comment, :config_id, :target_id, :schedule_id]

    def name=(val)
      name_will_change! unless @name == val
      @name = val
    end
    
    def comment=(val)
      comment_will_change! unless @comment == val
      @comment = val
    end
    
    def config_id=(v)
      config_id_will_change! unless @config_id == v
      @config_id = v
    end
    
    def target_id=(v)
      target_id_will_change! unless @target_id == v
      @target_id = v
    end
    
    def schedule_id=(v)
      schedule_id_will_change! unless @schedule_id == v
      @schedule_id = v
    end
    
    def schedule
      @schedule ||= pull_my_schedule
    end
    
    def schedule=(v)
      @schedule = v
      schedule_id = v ? v.id : nil 
    end
    
    def config
      @config ||= pull_my_config
    end
    
    def config=(val)
      @config = val
      config_id = val.id if val
    end
    
    def target
      @target ||= pull_my_target
    end
    
    def target=(val)
      @target = val
      target_id = val.id if val
    end
    
    def reports
      @reports ||= []
    end
    
    def create_or_update
          
      if schedule && schedule.changed?
        return unless schedule.save
        schedule_id = schedule.id
      end
      
      if config && config.changed?
        return unless config.save
        config_id = config.id
      end
       
      req = Nokogiri::XML::Builder.new { |xml|
        if @id
          xml.modify_task(:id => @id) {
            xml.name    { xml.text(@name) }    if name_changed?
            xml.comment { xml.text(@comment) } if comment_changed?
            xml.schedule(:id => @schedule_id)  if schedule_id && !schedule_id.blank? && schedule_id_changed?
          }
        else
          xml.create_task {
            xml.name    { xml.text(@name) }    if @name
            xml.comment { xml.text(@comment) } if @comment
            xml.config(:id => @config_id)
            xml.target(:id => @target_id)
            xml.schedule(:id => @schedule_id)  if @schedule_id && !@schedule_id.blank?
          }
        end
      }
      
      begin
        resp = VasTask.connection.send_receive(req.doc)
        @id = VasTask.extract_value_from("/create_task_response/@id", resp) unless @id
        reset_changes
        
        true
      rescue VasExceptions::CommandException => e
        errors[:command_failure] << e.message
        
        nil
      end
      
      
    end
    
    def delete_record
      req = Nokogiri::XML::Builder.new { |xml|
        xml.delete_task(:task_id => @id)
      }
      
      begin
        VasTask.connection.send_receive(req.doc)
        
        true
      rescue VasExceptions::CommandException => e
        errors[:command_failure] << e.message
        
        nil
      end
    end
    
    def start
      req = Nokogiri::XML::Builder.new { |xml|
        xml.resume_or_start_task(:task_id => @id)
      }
      
      VasTask.connection.send_receive(req.doc)
    end
    
    def progress=(val)
      @progress = val
    end
    
    def progress(refresh=false)
      if refresh == true || @progress == nil
        params = {:apply_overrides => 0, :sort_field => "name"}
        params[:task_id] = options[:task_id] if options[:task_id]
      
        req = Nokogiri::XML::Builder.new { |xml|
          xml.get_tasks
        }
        
        task = VasTask.connection.send_receive(req.doc)
      
        @progress = VasTaskProgress.from_xml_node task.at_xpath('/get_tasks_response/task/progress')
      end
      @progress
    end
    
    def pause
      req = Nokogiri::XML::Builder.new { |xml|
        xml.pause_task(:task_id => @id)
      }
      
      VasTask.connection.send_receive(req.doc)
    end
    
    def stop
      req = Nokogiri::XML::Builder.new { |xml|
        xml.stop_task(:task_id => @id)
      }
      
      VasTask.connection.send_receive(req.doc)
    end
              
    def self.get_all(options = {})
      params = {:apply_overrides => 0, :sort_field => "name"}
      
      params[:task_id] = options[:id] if options[:id]
    
      req = Nokogiri::XML::Builder.new { |xml|
        xml.get_tasks(params)
      }
      
      tasks = connection.send_receive(req.doc)
      
      ret = []
      tasks.xpath('//task').each { |t|
        ret << from_xml_node(t)
      }
      
      ret
    end
    
    def self.from_xml_node(node)
      t                = VasTask.new
      t.id             = extract_value_from("@id", node)
      t.name           = extract_value_from("name", node)
      t.comment        = extract_value_from("comment", node)
      t.status         = extract_value_from("status", node)
      if node.at_xpath("progress")
        t.progress = VasTaskProgress.from_xml_node(node.at_xpath("progress"))
      end
      t.times_run      = extract_value_from("report_count/finished", node).to_i
      t.last_report_id = extract_value_from("last_report/report/@id", node)
      t.config_id      = extract_value_from("config/@id", node)
      t.target_id      = extract_value_from("target/@id", node)
      node.xpath("reports/report").each { |xr|
        t.reports << VasReport.new({
          :id => extract_value_from("@id", xr),
          :started_at => extract_value_from("timestamp", xr)
        })
      }  
      
      t.reset_changes
      
      t
    end
   
  private
    def pull_my_config
      if @config_id && !@config_id.empty?
        VasConfig.get_all(:id => @config_id)[0]
      else
        nil
      end
    end
    
    def pull_my_target
      if @target_id && !@target_id.empty?
        VasTarget.get_all(:id => @target_id)[0]
      else
        nil
      end
    end
    
    def pull_my_schedule
      if @schedule_id && !@schedule_id.empty?
        VasSchedule.get_all(:id => @schedule_id)[0]
      else
        nil
      end
    end
  end
end
